@@ -10,7 +10,7 @@ import (
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
-	"gopkg.in/yaml.v3"
+	"go.yaml.in/yaml/v4"
 )
 
 func testGetWantedActionMetadata() *ActionMetadata {
@@ -102,6 +102,9 @@ func TestLocalActionsFindMetadataOK(t *testing.T) {
 	wantBranding.Branding.Icon = "edit"
 	wantBranding.Branding.Color = "white"
 
+	wantNode24 := testGetWantedActionMetadata()
+	wantNode24.Runs.Using = "node24"
+
 	tests := []struct {
 		spec string
 		want *ActionMetadata
@@ -151,6 +154,10 @@ func TestLocalActionsFindMetadataOK(t *testing.T) {
 			spec: "./branding",
 			want: wantBranding,
 		},
+		{
+			spec: "./node24",
+			want: wantNode24,
+		},
 	}
 
 	for _, tc := range tests {
