@@ -6,7 +6,7 @@ import (
 	"strconv"
 	"strings"
 
-	"gopkg.in/yaml.v3"
+	"go.yaml.in/yaml/v4"
 )
 
 // Pos represents position in the file.
@@ -328,7 +328,7 @@ func (e *WorkflowCallEvent) EventName() string {
 }
 
 // PermissionScope is struct for respective permission scope like "issues", "checks", ...
-// https://docs.github.com/en/actions/security-guides/automatic-token-authentication#permissions-for-the-github_token
+// https://docs.github.com/en/actions/security-for-github-actions/security-guides/automatic-token-authentication#permissions-for-the-github_token
 type PermissionScope struct {
 	// Name is name of the scope.
 	Name *String
@@ -555,13 +555,21 @@ func (a *RawYAMLArray) Pos() *Pos {
 }
 
 func (a *RawYAMLArray) String() string {
-	var b quotesBuilder
-	b.inner.WriteRune('[')
-	for _, v := range a.Elems {
-		b.append(v.String())
+	var b strings.Builder
+	b.WriteRune('[')
+
+	comma := false
+	for _, e := range a.Elems {
+		if comma {
+			b.WriteString(", ")
+		} else {
+			comma = true
+		}
+		b.WriteString(e.String())
 	}
-	b.inner.WriteRune(']')
-	return b.build()
+
+	b.WriteRune(']')
+	return b.String()
 }
 
 // RawYAMLString is raw YAML scalar value.
@@ -762,6 +770,19 @@ type Service struct {
 	Container *Container
 }
 
+// Services is a mapping from service ID to its configuration.
+// https://docs.github.com/en/actions/using-workflows/workflow-syntax-for-github-actions#jobsjob_idservices
+type Services struct {
+	// Value is a mapping from service ID to its Service instances. Keys are in lower case since
+	// they are case-insensitive.
+	Value map[string]*Service
+	// Expression is an expression assigned to the services mapping by ${{ }} placeholder. Otherwise
+	// this field is nil.
+	Expression *String
+	// Pos is a position in source.
+	Pos *Pos
+}
+
 // Output is output entry of the job.
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idoutputs
 type Output struct {
@@ -863,9 +884,9 @@ type Job struct {
 	ContinueOnError *Bool
 	// Container is container configuration to run the job.
 	Container *Container
-	// Services is map from service names to service configurations. Keys are in lower case since they are case-insensitive.
+	// Services is a services configurations.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idservices
-	Services map[string]*Service
+	Services *Services
 	// WorkflowCall is a workflow call by 'uses:'.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_iduses
 	WorkflowCall *WorkflowCall
