@@ -47,8 +47,8 @@ const (
 	ColorOptionKindNever
 )
 
-// LinterOptions is set of options for Linter instance. This struct should be created by user and
-// given to NewLinter factory function.
+// LinterOptions is set of options for Linter instance. This struct is used for NewLinter factory
+// function call. The zero value LinterOptions{} represents the default behavior.
 type LinterOptions struct {
 	// Verbose is flag if verbose log output is enabled.
 	Verbose bool
@@ -85,22 +85,29 @@ type LinterOptions struct {
 	// WorkingDir is a file path to the current working directory. When this value is empty, os.Getwd
 	// will be used to get a working directory.
 	WorkingDir string
+	// OnRulesCreated is a hook to add or remove the check rules. This function is called on checking
+	// every workflow files. Rules created by Linter instance are passed to the argument and the
+	// function should return the modified rules.
+	// Note that syntax errors may be reported even if this function returns nil or an empty slice.
+	OnRulesCreated func([]Rule) []Rule
 	// More options will come here
 }
 
 // Linter is struct to lint workflow files.
 type Linter struct {
-	projects      *Projects
-	out           io.Writer
-	logOut        io.Writer
-	logLevel      LogLevel
-	oneline       bool
-	shellcheck    string
-	pyflakes      string
-	ignorePats    []*regexp.Regexp
-	defaultConfig *Config
-	errFmt        *ErrorFormatter
-	cwd           string
+	projects       *Projects
+	out            io.Writer
+	logOut         io.Writer
+	logLevel       LogLevel
+	oneline        bool
+	shellcheck     string
+	pyflakes       string
+	ignorePats     IgnorePatterns
+	stdin          string
+	defaultConfig  *Config
+	errFmt         *ErrorFormatter
+	cwd            string
+	onRulesCreated func([]Rule) []Rule
 }
 
 // NewLinter creates a new Linter instance.
@@ -127,7 +134,7 @@ func NewLinter(out io.Writer, opts *LinterOptions) (*Linter, error) {
 		}
 	}
 
-	var lout io.Writer = io.Discard
+	lout := io.Discard
 	if opts.LogWriter != nil {
 		lout = opts.LogWriter
 	}
@@ -159,14 +166,19 @@ func NewLinter(out io.Writer, opts *LinterOptions) (*Linter, error) {
 		formatter = f
 	}
 
-	cwd := opts.WorkingDir
-	if cwd == "" {
-		if d, err := os.Getwd(); err == nil {
-			cwd = d
-		}
+	cwd := "."
+	if opts.WorkingDir != "" {
+		cwd = opts.WorkingDir
+	} else if d, err := os.Getwd(); err == nil {
+		cwd = d
 	}
 
-	return &Linter{
+	stdin := "<stdin>"
+	if opts.StdinFileName != "" {
+		stdin = opts.StdinFileName
+	}
+
+	l := &Linter{
 		NewProjects(),
 		out,
 		lout,
@@ -175,10 +187,15 @@ func NewLinter(out io.Writer, opts *LinterOptions) (*Linter, error) {
 		opts.Shellcheck,
 		opts.Pyflakes,
 		ignore,
+		stdin,
 		cfg,
 		formatter,
 		cwd,
-	}, nil
+		opts.OnRulesCreated,
+	}
+
+	l.debug("Create a Linter instance with option %#v", opts)
+	return l, nil
 }
 
 func (l *Linter) log(args ...interface{}) {
@@ -204,43 +221,63 @@ func (l *Linter) debugWriter() io.Writer {
 	return l.logOut
 }
 
-// GenerateDefaultConfig generates default config file at ".github/actionlint.yaml" in project
-// which the given directory path belongs to.
+// GenerateDefaultConfig generates default config file at ".github/actionlint.yaml" in the project
+// which the given directory path belongs to. When the directory path is empty, the current directory
+// will be used instead.
 func (l *Linter) GenerateDefaultConfig(dir string) error {
+	if dir == "" {
+		dir = l.cwd
+	}
+
 	l.log("Generating default actionlint.yaml in repository:", dir)
 
-	p := l.projects.At(dir)
-	if p == nil {
+	proj, err := l.projects.At(dir)
+	if err != nil {
+		return err
+	}
+	if proj == nil {
 		return errors.New("project is not found. check current project is initialized as Git repository and \".github/workflows\" directory exists")
 	}
 
-	path := filepath.Join(p.RootDir(), ".github", "actionlint.yaml")
-	if _, err := os.Stat(path); err == nil {
-		return fmt.Errorf("config file already exists at %q", path)
+	d := filepath.Join(proj.RootDir(), ".github")
+	for _, f := range []string{"actionlint.yaml", "actionlint.yml"} {
+		p := filepath.Join(d, f)
+		if _, err := os.Stat(p); err == nil {
+			return fmt.Errorf("config file already exists at %q", p)
+		}
 	}
 
-	if err := writeDefaultConfigFile(path); err != nil {
+	p := filepath.Join(d, "actionlint.yaml")
+	if err := writeDefaultConfigFile(p); err != nil {
 		return err
 	}
 
-	fmt.Fprintf(l.out, "Config file was generated at %q\n", path)
+	fmt.Fprintf(l.out, "Config file was generated at %q\n", p)
 	return nil
 }
 
-// LintRepository lints YAML workflow files and outputs the errors to given writer. It finds the nearest
-// `.github/workflows` directory based on `dir` and applies lint rules to all YAML workflow files
-// under the directory.
+// LintRepository lints YAML workflow files and outputs the errors to given writer. It finds the
+// nearest `.github/workflows` directory based on `dir` and applies lint rules to all YAML workflow
+// files under the directory. When the directory path is empty, the current working directory will
+// be used instead.
 func (l *Linter) LintRepository(dir string) ([]*Error, error) {
+	if dir == "" {
+		dir = l.cwd
+	}
+
 	l.log("Linting all workflow files in repository:", dir)
 
-	proj := l.projects.At(dir)
-	if proj == nil {
+	p, err := l.projects.At(dir)
+	if err != nil {
+		return nil, err
+	}
+	if p == nil {
 		return nil, fmt.Errorf("no project was found in any parent directories of %q. check workflows directory is put correctly in your Git repository", dir)
 	}
 
-	l.log("Detected project:", proj.RootDir())
-	wd := proj.WorkflowsDir()
-	return l.LintDir(wd, proj)
+	l.log("Detected project:", p.RootDir())
+	wd := p.WorkflowsDir()
+	return l.LintDir(wd, p)
 }
 
 // LintDir lints all YAML workflow files in the given directory recursively.
@@ -287,8 +324,9 @@ func (l *Linter) LintFiles(filepaths []string, project *Project) ([]*Error, erro
 	l.log("Linting", n, "files")
 
 	cwd := l.cwd
-	proc := newConcurrentProcess(runtime.NumCPU())
-	sema := semaphore.NewWeighted(int64(runtime.NumCPU()))
+	cpus := runtime.NumCPU()
+	proc := newConcurrentProcess(cpus)
+	sema := semaphore.NewWeighted(int64(cpus))
 	ctx := context.Background()
 	dbg := l.debugWriter()
 	acf := NewLocalActionsCacheFactory(dbg)
@@ -309,14 +347,18 @@ func (l *Linter) LintFiles(filepaths []string, project *Project) ([]*Error, erro
 	for i := range ws {
 		// Each element of ws is accessed by single goroutine so mutex is unnecessary
 		w := &ws[i]
-		p := project
-		if p == nil {
+		proj := project
+		if proj == nil {
 			// This method modifies state of l.projects so it cannot be called in parallel.
 			// Before entering goroutine, resolve project instance.
-			p = l.projects.At(w.path)
+			p, err := l.projects.At(w.path)
+			if err != nil {
+				return nil, err
+			}
+			proj = p
 		}
-		ac := acf.GetCache(p) // #173
-		rwc := rwcf.GetCache(p)
+		ac := acf.GetCache(proj) // #173
+		rwc := rwcf.GetCache(proj)
 
 		eg.Go(func() error {
 			// Bound concurrency on reading files to avoid "too many files to open" error (issue #3)
@@ -332,7 +374,7 @@ func (l *Linter) LintFiles(filepaths []string, project *Project) ([]*Error, erro
 					w.path = r // Use relative path if possible
 				}
 			}
-			errs, err := l.check(w.path, src, p, proc, ac, rwc)
+			errs, err := l.check(w.path, src, proj, proc, ac, rwc)
 			if err != nil {
 				return fmt.Errorf("fatal error while checking %s: %w", w.path, err)
 			}
@@ -342,11 +384,18 @@ func (l *Linter) LintFiles(filepaths []string, project *Project) ([]*Error, erro
 		})
 	}
 
-	proc.wait()
 	if err := eg.Wait(); err != nil {
 		return nil, err
 	}
 
+	// Ensure that all processes finish. `proc.wait()` must be called after `eg.Wait()`.
+	// Calling `WaitGroup.Add` after `WaitGroup.Wait` can cause a race condition (specifically when
+	// increasing the group count from 0 to 1 and calling `Wait` and at the same time).
+	// `WaitGroup.Add` is called in `proc.run()` and `WaitGroup.Wait` is called in `proc.wait()`.
+	// After traversing all workflows, `proc.run()` is no longer called so `proc.wait()` can be
+	// called safely.
+	proc.wait()
+
 	total := 0
 	for i := range ws {
 		total += len(ws[i].errs)
@@ -382,7 +431,11 @@ func (l *Linter) LintFiles(filepaths []string, project *Project) ([]*Error, erro
 // parameter can be nil. In the case, the project is detected from the given path.
 func (l *Linter) LintFile(path string, project *Project) ([]*Error, error) {
 	if project == nil {
-		project = l.projects.At(path)
+		p, err := l.projects.At(path)
+		if err != nil {
+			return nil, err
+		}
+		project = p
 	}
 
 	src, err := os.ReadFile(path)
@@ -414,14 +467,29 @@ func (l *Linter) LintFile(path string, project *Project) ([]*Error, error) {
 	return errs, err
 }
 
+// LintStdin lints the content read from STDIN. The stdin parameter is a reader to read from STDIN,
+// which is usually os.Stdin. The file name is determined by LinterOptions.StdinFileName. When the
+// option is empty, "<stdin>" is the default value.
+func (l *Linter) LintStdin(stdin io.Reader) ([]*Error, error) {
+	l.log("Reading the input from stdin")
+	b, err := io.ReadAll(stdin)
+	if err != nil {
+		return nil, fmt.Errorf("could not read stdin: %w", err)
+	}
+	return l.Lint(l.stdin, b, nil)
+}
+
 // Lint lints YAML workflow file content given as byte slice. The path parameter is used as file
-// path where the content came from. Setting "<stdin>" to path parameter indicates the output came
-// from STDIN.
+// path where the content came from.
 // When nil is passed to the project parameter, it tries to find the project from the path parameter.
 func (l *Linter) Lint(path string, content []byte, project *Project) ([]*Error, error) {
 	if project == nil && path != "<stdin>" {
 		if _, err := os.Stat(path); !errors.Is(err, fs.ErrNotExist) {
-			project = l.projects.At(path)
+			p, err := l.projects.At(path)
+			if err != nil {
+				return nil, err
+			}
+			project = p
 		}
 	}
 	proc := newConcurrentProcess(runtime.NumCPU())
@@ -464,13 +532,10 @@ func (l *Linter) check(
 
 	var cfg *Config
 	if l.defaultConfig != nil {
+		// `-config-file` option has higher priority than repository config file
 		cfg = l.defaultConfig
 	} else if project != nil {
-		c, err := project.LoadConfig()
-		if err != nil {
-			return nil, err
-		}
-		cfg = c
+		cfg = project.Config()
 	}
 	if cfg != nil {
 		l.debug("Config: %#v", cfg)
@@ -503,6 +568,7 @@ func (l *Linter) check(
 			NewRuleWorkflowCall(path, localReusableWorkflows),
 			NewRuleExpression(localActions, localReusableWorkflows),
 			NewRuleDeprecatedCommands(),
+			NewRuleIfCond(),
 		}
 		if l.shellcheck != "" {
 			r, err := NewRuleShellcheck(l.shellcheck, proc)
@@ -524,6 +590,9 @@ func (l *Linter) check(
 		} else {
 			l.log("Rule \"pyflakes\" was disabled since pyflakes command name was empty")
 		}
+		if l.onRulesCreated != nil {
+			rules = l.onRulesCreated(rules)
+		}
 
 		v := NewVisitor()
 		for _, rule := range rules {
@@ -542,7 +611,7 @@ func (l *Linter) check(
 		}
 
 		if err := v.Visit(w); err != nil {
-			l.debug("error occurred while visiting workflow syntax tree: %v", err)
+			l.debug("Error occurred while visiting workflow syntax tree: %v", err)
 			return nil, err
 		}
 
@@ -551,21 +620,15 @@ func (l *Linter) check(
 			l.debug("%s found %d errors", rule.Name(), len(errs))
 			all = append(all, errs...)
 		}
+
+		if l.errFmt != nil {
+			for _, rule := range rules {
+				l.errFmt.RegisterRule(rule)
+			}
+		}
 	}
 
-	if len(l.ignorePats) > 0 {
-		filtered := make([]*Error, 0, len(all))
-	Loop:
-		for _, err := range all {
-			for _, pat := range l.ignorePats {
-				if pat.MatchString(err.Message) {
-					continue Loop
-				}
-			}
-			filtered = append(filtered, err)
-		}
-		all = filtered
-	}
+	all = l.filterErrors(all, cfg.PathConfigs(path))
 
 	for _, err := range all {
 		err.Filepath = path // Populate filename in the error
@@ -581,6 +644,32 @@ func (l *Linter) check(
 	return all, nil
 }
 
+func (l *Linter) filterErrors(errs []*Error, cfgs []PathConfig) []*Error {
+	if len(l.ignorePats) == 0 && len(cfgs) == 0 {
+		return errs
+	}
+
+	filtered := make([]*Error, 0, len(errs))
+Loop:
+	for _, err := range errs {
+		if l.ignorePats.Match(err) {
+			l.debug("Error %q is ignored due to -ignore command line option", err.Message)
+			continue Loop
+		}
+		for _, c := range cfgs {
+			if c.Ignore.Match(err) {
+				l.debug("Error %q is ignored due to the \"ignore\" config in the config file", err.Message)
+				continue Loop
+			}
+		}
+		filtered = append(filtered, err)
+	}
+	if len(filtered) != len(errs) {
+		l.log("Filtered", len(errs)-len(filtered), "error(s) due to \"-ignore\" command line option and \"ignore\" configuration")
+	}
+	return filtered
+}
+
 func (l *Linter) printErrors(errs []*Error, src []byte) {
 	if l.oneline {
 		src = nil
