@@ -2,14 +2,14 @@ package actionlint
 
 import (
 	"fmt"
-	"os"
-	"regexp"
+	"math"
 	"strconv"
 	"strings"
 
-	"gopkg.in/yaml.v3"
+	"go.yaml.in/yaml/v4"
 )
 
+// https://pkg.go.dev/go.yaml.in/yaml/v4#Kind
 func nodeKindName(k yaml.Kind) string {
 	switch k {
 	case yaml.DocumentNode:
@@ -23,10 +23,7 @@ func nodeKindName(k yaml.Kind) string {
 	case yaml.AliasNode:
 		return "alias"
 	default:
-		if os.Getenv("ACTIONLINT_DEBUG") != "" {
-			return "unknown"
-		}
-		panic("unreachable")
+		panic(fmt.Sprintf("unreachable: unknown YAML kind: %v", k))
 	}
 }
 
@@ -44,7 +41,8 @@ func newString(n *yaml.Node) *String {
 }
 
 type workflowKeyVal struct {
-	id  string // Key in lower case since workflow keys are case-insensitive
+	// id is used for comparing keys. When the key is case insensitive, this field is in lower case.
+	id  string
 	key *String
 	val *yaml.Node
 }
@@ -119,12 +117,7 @@ func (p *parser) missingExpression(n *yaml.Node, expecting string) {
 }
 
 func (p *parser) parseExpression(n *yaml.Node, expecting string) *String {
-	s := strings.TrimSpace(n.Value)
-	if !strings.HasPrefix(s, "${{") || !strings.HasSuffix(s, "}}") {
-		p.missingExpression(n, expecting)
-		return nil
-	}
-	if strings.Count(n.Value, "${{") != 1 {
+	if !isExprAssigned(n.Value) {
 		p.missingExpression(n, expecting)
 		return nil
 	}
@@ -135,8 +128,7 @@ func (p *parser) mayParseExpression(n *yaml.Node) *String {
 	if n.Tag != "!!str" {
 		return nil
 	}
-	s := strings.TrimSpace(n.Value)
-	if !strings.HasPrefix(s, "${{") || !strings.HasSuffix(s, "}}") || strings.Count(n.Value, "${{") != 1 {
+	if !isExprAssigned(n.Value) {
 		return nil
 	}
 	return newString(n)
@@ -204,6 +196,9 @@ func (p *parser) parseInt(n *yaml.Node) *Int {
 
 	if n.Tag == "!!str" {
 		e := p.parseExpression(n, "integer literal")
+		if e == nil {
+			return nil
+		}
 		return &Int{
 			Expression: e,
 			Pos:        posAt(n),
@@ -230,6 +225,9 @@ func (p *parser) parseFloat(n *yaml.Node) *Float {
 
 	if n.Tag == "!!str" {
 		e := p.parseExpression(n, "float number literal")
+		if e == nil {
+			return nil
+		}
 		return &Float{
 			Expression: e,
 			Pos:        posAt(n),
@@ -237,7 +235,7 @@ func (p *parser) parseFloat(n *yaml.Node) *Float {
 	}
 
 	f, err := strconv.ParseFloat(n.Value, 64)
-	if err != nil {
+	if err != nil || math.IsNaN(f) {
 		p.errorf(n, "invalid float value: %q: %s", n.Value, err.Error())
 		return nil
 	}
@@ -248,7 +246,7 @@ func (p *parser) parseFloat(n *yaml.Node) *Float {
 	}
 }
 
-func (p *parser) parseMapping(what string, n *yaml.Node, allowEmpty bool) []workflowKeyVal {
+func (p *parser) parseMapping(what string, n *yaml.Node, allowEmpty, caseSensitive bool) []workflowKeyVal {
 	isNull := isNull(n)
 
 	if !isNull && n.Kind != yaml.MappingNode {
@@ -270,15 +268,22 @@ func (p *parser) parseMapping(what string, n *yaml.Node, allowEmpty bool) []work
 			continue
 		}
 
-		// Keys of mappings are case insensitive. For example, following matrix is invalid.
-		// matrix:
-		//   foo: [1, 2, 3]
-		//   FOO: [1, 2, 3]
-		// To detect case insensitive duplicate keys, we use lowercase keys always
-		id := strings.ToLower(k.Value)
+		id := k.Value
+		if !caseSensitive {
+			// Keys of mappings are sometimes case insensitive. For example, following matrix is invalid.
+			//   matrix:
+			//     foo: [1, 2, 3]
+			//     FOO: [1, 2, 3]
+			// To detect case insensitive duplicate keys, we use lowercase keys
+			id = strings.ToLower(id)
+		}
 
 		if pos, ok := keys[id]; ok {
-			p.errorfAt(k.Pos, "key %q is duplicated in %s. previously defined at %s. note that key names are case insensitive", k.Value, what, pos.String())
+			var note string
+			if !caseSensitive {
+				note = ". note that this key is case insensitive"
+			}
+			p.errorfAt(k.Pos, "key %q is duplicated in %s. previously defined at %s%s", k.Value, what, pos.String(), note)
 			continue
 		}
 		m = append(m, workflowKeyVal{id, k, n.Content[i+1]})
@@ -292,8 +297,8 @@ func (p *parser) parseMapping(what string, n *yaml.Node, allowEmpty bool) []work
 	return m
 }
 
-func (p *parser) parseSectionMapping(sec string, n *yaml.Node, allowEmpty bool) []workflowKeyVal {
-	return p.parseMapping(fmt.Sprintf("%q section", sec), n, allowEmpty)
+func (p *parser) parseSectionMapping(sec string, n *yaml.Node, allowEmpty, caseSensitive bool) []workflowKeyVal {
+	return p.parseMapping(fmt.Sprintf("%q section", sec), n, allowEmpty, caseSensitive)
 }
 
 func (p *parser) parseScheduleEvent(pos *Pos, n *yaml.Node) *ScheduledEvent {
@@ -303,7 +308,7 @@ func (p *parser) parseScheduleEvent(pos *Pos, n *yaml.Node) *ScheduledEvent {
 
 	cron := make([]*String, 0, len(n.Content))
 	for _, c := range n.Content {
-		m := p.parseMapping("element of \"schedule\" section", c, false)
+		m := p.parseMapping("element of \"schedule\" section", c, false, true)
 		if len(m) != 1 || m[0].id != "cron" {
 			p.error(c, "element of \"schedule\" section must be mapping and must contain one key \"cron\"")
 			continue
@@ -317,17 +322,17 @@ func (p *parser) parseScheduleEvent(pos *Pos, n *yaml.Node) *ScheduledEvent {
 	return &ScheduledEvent{cron, pos}
 }
 
-// https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows#workflow_dispatch
+// https://docs.github.com/en/actions/reference/workflows-and-actions/events-that-trigger-workflows#workflow_dispatch
 func (p *parser) parseWorkflowDispatchEvent(pos *Pos, n *yaml.Node) *WorkflowDispatchEvent {
 	ret := &WorkflowDispatchEvent{Pos: pos}
 
-	for _, kv := range p.parseSectionMapping("workflow_dispatch", n, true) {
+	for _, kv := range p.parseSectionMapping("workflow_dispatch", n, true, true) {
 		if kv.id != "inputs" {
 			p.unexpectedKey(kv.key, "workflow_dispatch", []string{"inputs"})
 			continue
 		}
 
-		inputs := p.parseSectionMapping("inputs", kv.val, true)
+		inputs := p.parseSectionMapping("inputs", kv.val, true, false)
 		ret.Inputs = make(map[string]*DispatchInput, len(inputs))
 		for _, input := range inputs {
 			name, spec := input.key, input.val
@@ -335,10 +340,9 @@ func (p *parser) parseWorkflowDispatchEvent(pos *Pos, n *yaml.Node) *WorkflowDis
 			var desc *String
 			var req *Bool
 			var def *String
-			var ty WorkflowDispatchEventInputType = WorkflowDispatchEventInputTypeNone
 			var opts []*String
-
-			for _, attr := range p.parseMapping("input settings of workflow_dispatch event", spec, true) {
+			ty := WorkflowDispatchEventInputTypeNone
+			for _, attr := range p.parseMapping("input settings of workflow_dispatch event", spec, true, true) {
 				switch attr.id {
 				case "description":
 					desc = p.parseString(attr.val, true)
@@ -353,6 +357,8 @@ func (p *parser) parseWorkflowDispatchEvent(pos *Pos, n *yaml.Node) *WorkflowDis
 					switch attr.val.Value {
 					case "string":
 						ty = WorkflowDispatchEventInputTypeString
+					case "number":
+						ty = WorkflowDispatchEventInputTypeNumber
 					case "boolean":
 						ty = WorkflowDispatchEventInputTypeBoolean
 					case "choice":
@@ -360,7 +366,7 @@ func (p *parser) parseWorkflowDispatchEvent(pos *Pos, n *yaml.Node) *WorkflowDis
 					case "environment":
 						ty = WorkflowDispatchEventInputTypeEnvironment
 					default:
-						p.errorf(attr.val, "input type of workflow_dispatch event must be one of \"string\", \"boolean\", \"choice\", \"environment\" but got %q", attr.val.Value)
+						p.errorf(attr.val, `input type of workflow_dispatch event must be one of "string", "number", "boolean", "choice", "environment" but got %q`, attr.val.Value)
 					}
 				case "options":
 					opts = p.parseStringSequence("options", attr.val, false, false)
@@ -383,12 +389,12 @@ func (p *parser) parseWorkflowDispatchEvent(pos *Pos, n *yaml.Node) *WorkflowDis
 	return ret
 }
 
-// https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows#repository_dispatch
+// https://docs.github.com/en/actions/reference/workflows-and-actions/events-that-trigger-workflows#repository_dispatch
 func (p *parser) parseRepositoryDispatchEvent(pos *Pos, n *yaml.Node) *RepositoryDispatchEvent {
 	ret := &RepositoryDispatchEvent{Pos: pos}
 
 	// Note: Omitting 'types' is ok. In the case, all types trigger the workflow
-	for _, kv := range p.parseSectionMapping("repository_dispatch", n, true) {
+	for _, kv := range p.parseSectionMapping("repository_dispatch", n, true, true) {
 		if kv.id == "types" {
 			ret.Types = p.parseStringOrStringSequence("types", kv.val, false, false)
 		} else {
@@ -416,7 +422,7 @@ func (p *parser) parseWebhookEvent(name *String, n *yaml.Node) *WebhookEvent {
 	// > undefined Git ref.
 	//
 	// https://github.community/t/using-on-push-tags-ignore-and-paths-ignore-together/16931
-	for _, kv := range p.parseSectionMapping(name.Value, n, true) {
+	for _, kv := range p.parseSectionMapping(name.Value, n, true, true) {
 		// Note: Glob pattern cannot be empty, but it is checked by 'glob' rule with better error
 		// message. So parser allows empty patterns here.
 		switch kv.id {
@@ -453,23 +459,23 @@ func (p *parser) parseWebhookEvent(name *String, n *yaml.Node) *WebhookEvent {
 	return ret
 }
 
-// - https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows#workflow-reuse-events
+// - https://docs.github.com/en/actions/reference/workflows-and-actions/events-that-trigger-workflows#workflow-reuse-events
 // - https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#onworkflow_callinputs
 // - https://docs.github.com/en/actions/learn-github-actions/reusing-workflows
 func (p *parser) parseWorkflowCallEvent(pos *Pos, n *yaml.Node) *WorkflowCallEvent {
 	ret := &WorkflowCallEvent{Pos: pos}
 
-	for _, kv := range p.parseSectionMapping("workflow_call", n, true) {
+	for _, kv := range p.parseSectionMapping("workflow_call", n, true, true) {
 		switch kv.id {
 		case "inputs":
-			inputs := p.parseSectionMapping("inputs", kv.val, true)
+			inputs := p.parseSectionMapping("inputs", kv.val, true, false)
 			ret.Inputs = make([]*WorkflowCallEventInput, 0, len(inputs))
 			for _, kv := range inputs {
 				name, spec := kv.key, kv.val
 				input := &WorkflowCallEventInput{Name: name, ID: kv.id}
 				sawType := false
 
-				for _, attr := range p.parseMapping("input of workflow_call event", spec, true) {
+				for _, attr := range p.parseMapping("input of workflow_call event", spec, true, true) {
 					switch attr.id {
 					case "description":
 						input.Description = p.parseString(attr.val, true)
@@ -501,13 +507,13 @@ func (p *parser) parseWorkflowCallEvent(pos *Pos, n *yaml.Node) *WorkflowCallEve
 				ret.Inputs = append(ret.Inputs, input)
 			}
 		case "secrets":
-			secrets := p.parseSectionMapping("secrets", kv.val, true)
+			secrets := p.parseSectionMapping("secrets", kv.val, true, false)
 			ret.Secrets = make(map[string]*WorkflowCallEventSecret, len(secrets))
 			for _, kv := range secrets {
 				name, spec := kv.key, kv.val
 				secret := &WorkflowCallEventSecret{Name: name}
 
-				for _, attr := range p.parseMapping("secret of workflow_call event", spec, true) {
+				for _, attr := range p.parseMapping("secret of workflow_call event", spec, true, true) {
 					switch attr.id {
 					case "description":
 						secret.Description = p.parseString(attr.val, true)
@@ -521,13 +527,13 @@ func (p *parser) parseWorkflowCallEvent(pos *Pos, n *yaml.Node) *WorkflowCallEve
 				ret.Secrets[kv.id] = secret
 			}
 		case "outputs":
-			outputs := p.parseSectionMapping("outputs", kv.val, true)
+			outputs := p.parseSectionMapping("outputs", kv.val, true, false)
 			ret.Outputs = make(map[string]*WorkflowCallEventOutput, len(outputs))
 			for _, kv := range outputs {
 				name, spec := kv.key, kv.val
 				output := &WorkflowCallEventOutput{Name: name}
 
-				for _, attr := range p.parseMapping("output of workflow_call event", spec, true) {
+				for _, attr := range p.parseMapping("output of workflow_call event", spec, true, true) {
 					switch attr.id {
 					case "description":
 						output.Description = p.parseString(attr.val, true)
@@ -545,7 +551,7 @@ func (p *parser) parseWorkflowCallEvent(pos *Pos, n *yaml.Node) *WorkflowCallEve
 				ret.Outputs[kv.id] = output
 			}
 		default:
-			p.unexpectedKey(kv.key, "workflow_call", []string{"inputs", "secrets"})
+			p.unexpectedKey(kv.key, "workflow_call", []string{"inputs", "secrets", "outputs"})
 		}
 	}
 
@@ -584,7 +590,7 @@ func (p *parser) parseEvents(pos *Pos, n *yaml.Node) []Event {
 			}
 		}
 	case yaml.MappingNode:
-		kvs := p.parseSectionMapping("on", n, false)
+		kvs := p.parseSectionMapping("on", n, false, true)
 		ret := make([]Event, 0, len(kvs))
 
 		for _, kv := range kvs {
@@ -640,7 +646,7 @@ func (p *parser) parsePermissions(pos *Pos, n *yaml.Node) *Permissions {
 	if n.Kind == yaml.ScalarNode {
 		ret.All = p.parseString(n, false)
 	} else {
-		m := p.parseSectionMapping("permissions", n, true)
+		m := p.parseSectionMapping("permissions", n, true, false) // XXX: Is the permission scope case insensitive?
 		scopes := make(map[string]*PermissionScope, len(m))
 		for _, kv := range m {
 			scopes[kv.id] = &PermissionScope{
@@ -662,7 +668,7 @@ func (p *parser) parseEnv(n *yaml.Node) *Env {
 		}
 	}
 
-	m := p.parseMapping("env", n, false)
+	m := p.parseMapping("env", n, false, false)
 	vars := make(map[string]*EnvVar, len(m))
 
 	for _, kv := range m {
@@ -679,14 +685,14 @@ func (p *parser) parseEnv(n *yaml.Node) *Env {
 func (p *parser) parseDefaults(pos *Pos, n *yaml.Node) *Defaults {
 	ret := &Defaults{Pos: pos}
 
-	for _, kv := range p.parseSectionMapping("defaults", n, false) {
+	for _, kv := range p.parseSectionMapping("defaults", n, false, true) {
 		if kv.id != "run" {
 			p.unexpectedKey(kv.key, "defaults", []string{"run"})
 			continue
 		}
 		ret.Run = &DefaultsRun{Pos: kv.key.Pos}
 
-		for _, attr := range p.parseSectionMapping("run", kv.val, false) {
+		for _, attr := range p.parseSectionMapping("run", kv.val, false, true) {
 			switch attr.id {
 			case "shell":
 				ret.Run.Shell = p.parseString(attr.val, false)
@@ -713,7 +719,7 @@ func (p *parser) parseConcurrency(pos *Pos, n *yaml.Node) *Concurrency {
 		ret.Group = p.parseString(n, false)
 	} else {
 		groupFound := false
-		for _, kv := range p.parseSectionMapping("concurrency", n, false) {
+		for _, kv := range p.parseSectionMapping("concurrency", n, false, true) {
 			switch kv.id {
 			case "group":
 				ret.Group = p.parseString(kv.val, false)
@@ -725,7 +731,7 @@ func (p *parser) parseConcurrency(pos *Pos, n *yaml.Node) *Concurrency {
 			}
 		}
 		if !groupFound {
-			p.error(n, "group name is missing in \"concurrency\" section")
+			p.errorAt(pos, "group name is missing in \"concurrency\" section")
 		}
 	}
 
@@ -740,7 +746,7 @@ func (p *parser) parseEnvironment(pos *Pos, n *yaml.Node) *Environment {
 		ret.Name = p.parseString(n, false)
 	} else {
 		nameFound := false
-		for _, kv := range p.parseSectionMapping("environment", n, false) {
+		for _, kv := range p.parseSectionMapping("environment", n, false, true) {
 			switch kv.id {
 			case "name":
 				ret.Name = p.parseString(kv.val, false)
@@ -752,7 +758,7 @@ func (p *parser) parseEnvironment(pos *Pos, n *yaml.Node) *Environment {
 			}
 		}
 		if !nameFound {
-			p.error(n, "name is missing in \"environment\" section")
+			p.errorAt(pos, "name is missing in \"environment\" section")
 		}
 	}
 
@@ -761,7 +767,7 @@ func (p *parser) parseEnvironment(pos *Pos, n *yaml.Node) *Environment {
 
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idoutputs
 func (p *parser) parseOutputs(n *yaml.Node) map[string]*Output {
-	outputs := p.parseSectionMapping("outputs", n, false)
+	outputs := p.parseSectionMapping("outputs", n, false, false)
 	ret := make(map[string]*Output, len(outputs))
 	for _, output := range outputs {
 		ret[output.id] = &Output{
@@ -786,7 +792,7 @@ func (p *parser) parseRawYAMLValue(n *yaml.Node) RawYAMLValue {
 		}
 		return &RawYAMLArray{vs, posAt(n)}
 	case yaml.MappingNode:
-		parsed := p.parseMapping("matrix row value", n, true)
+		parsed := p.parseMapping("matrix row value", n, true, false)
 		m := make(map[string]RawYAMLValue, len(parsed))
 		for _, kv := range parsed {
 			if v := p.parseRawYAMLValue(kv.val); v != nil {
@@ -822,7 +828,7 @@ func (p *parser) parseMatrixCombinations(sec string, n *yaml.Node) *MatrixCombin
 			continue
 		}
 
-		kvs := p.parseMapping(fmt.Sprintf("element in %q section", sec), c, false)
+		kvs := p.parseMapping(fmt.Sprintf("element in %q section", sec), c, false, false)
 		assigns := make(map[string]*MatrixAssign, len(kvs))
 		for _, kv := range kvs {
 			if v := p.parseRawYAMLValue(kv.val); v != nil {
@@ -845,7 +851,7 @@ func (p *parser) parseMatrix(pos *Pos, n *yaml.Node) *Matrix {
 
 	ret := &Matrix{Pos: pos, Rows: make(map[string]*MatrixRow)}
 
-	for _, kv := range p.parseSectionMapping("matrix", n, false) {
+	for _, kv := range p.parseSectionMapping("matrix", n, false, false) {
 		switch kv.id {
 		case "include":
 			ret.Include = p.parseMatrixCombinations("include", kv.val)
@@ -880,18 +886,27 @@ func (p *parser) parseMatrix(pos *Pos, n *yaml.Node) *Matrix {
 	return ret
 }
 
+// https://docs.github.com/en/actions/using-workflows/workflow-syntax-for-github-actions#jobsjob_idstrategymax-parallel
+func (p *parser) parseMaxParallel(n *yaml.Node) *Int {
+	i := p.parseInt(n)
+	if i != nil && i.Expression == nil && i.Value <= 0 {
+		p.errorf(n, "value at \"max-parallel\" must be greater than zero: %v", i.Value)
+	}
+	return i
+}
+
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idstrategy
 func (p *parser) parseStrategy(pos *Pos, n *yaml.Node) *Strategy {
 	ret := &Strategy{Pos: pos}
 
-	for _, kv := range p.parseSectionMapping("strategy", n, false) {
+	for _, kv := range p.parseSectionMapping("strategy", n, false, true) {
 		switch kv.id {
 		case "matrix":
 			ret.Matrix = p.parseMatrix(kv.key.Pos, kv.val)
 		case "fail-fast":
 			ret.FailFast = p.parseBool(kv.val)
 		case "max-parallel":
-			ret.MaxParallel = p.parseInt(kv.val)
+			ret.MaxParallel = p.parseMaxParallel(kv.val)
 		default:
 			p.unexpectedKey(kv.key, "strategy", []string{"matrix", "fail-fast", "max-parallel"})
 		}
@@ -908,13 +923,13 @@ func (p *parser) parseContainer(sec string, pos *Pos, n *yaml.Node) *Container {
 		// When you only specify a container image, you can omit the image keyword.
 		ret.Image = p.parseString(n, false)
 	} else {
-		for _, kv := range p.parseSectionMapping(sec, n, false) {
+		for _, kv := range p.parseSectionMapping(sec, n, false, true) {
 			switch kv.id {
 			case "image":
 				ret.Image = p.parseString(kv.val, false)
 			case "credentials":
 				cred := &Credentials{Pos: kv.key.Pos}
-				for _, c := range p.parseSectionMapping("credentials", kv.val, false) {
+				for _, c := range p.parseSectionMapping("credentials", kv.val, false, true) {
 					switch c.id {
 					case "username":
 						cred.Username = p.parseString(c.val, false)
@@ -953,12 +968,39 @@ func (p *parser) parseContainer(sec string, pos *Pos, n *yaml.Node) *Container {
 	return ret
 }
 
+func (p *parser) parseServices(n *yaml.Node) *Services {
+	ret := &Services{Pos: posAt(n)}
+	if e := p.mayParseExpression(n); e != nil {
+		ret.Expression = e
+	} else {
+		services := p.parseSectionMapping("services", n, false, false) // XXX: Is the key case-insensitive?
+		ss := make(map[string]*Service, len(services))
+		for _, s := range services {
+			ss[s.id] = &Service{
+				Name:      s.key,
+				Container: p.parseContainer("services", s.key.Pos, s.val),
+			}
+		}
+		ret.Value = ss
+	}
+	return ret
+}
+
+// https://docs.github.com/en/actions/using-workflows/workflow-syntax-for-github-actions#jobsjob_idtimeout-minutes
+func (p *parser) parseTimeoutMinutes(n *yaml.Node) *Float {
+	f := p.parseFloat(n)
+	if f != nil && f.Expression == nil && f.Value <= 0.0 {
+		p.errorf(n, "value at \"timeout-minutes\" must be greater than zero: %v", f.Value)
+	}
+	return f
+}
+
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobsjob_idsteps
 func (p *parser) parseStep(n *yaml.Node) *Step {
 	ret := &Step{Pos: posAt(n)}
 	var workDir *String
 
-	for _, kv := range p.parseMapping("element of \"steps\" section", n, false) {
+	for _, kv := range p.parseMapping("element of \"steps\" section", n, false, true) {
 		switch kv.id {
 		case "id":
 			ret.ID = p.parseString(kv.val, false)
@@ -971,7 +1013,7 @@ func (p *parser) parseStep(n *yaml.Node) *Step {
 		case "continue-on-error":
 			ret.ContinueOnError = p.parseBool(kv.val)
 		case "timeout-minutes":
-			ret.TimeoutMinutes = p.parseFloat(kv.val)
+			ret.TimeoutMinutes = p.parseTimeoutMinutes(kv.val)
 		case "uses", "with":
 			var exec *ExecAction
 			if ret.Exec == nil {
@@ -986,7 +1028,7 @@ func (p *parser) parseStep(n *yaml.Node) *Step {
 				exec.Uses = p.parseString(kv.val, false)
 			} else {
 				// kv.key == "with"
-				with := p.parseSectionMapping("with", kv.val, false)
+				with := p.parseSectionMapping("with", kv.val, false, false)
 				exec.Inputs = make(map[string]*Input, len(with))
 				for _, input := range with {
 					switch input.id {
@@ -1091,7 +1133,7 @@ func (p *parser) parseRunsOn(n *yaml.Node) *Runner {
 	}
 
 	r := &Runner{}
-	for _, kv := range p.parseSectionMapping("runs-on", n, false) {
+	for _, kv := range p.parseSectionMapping("runs-on", n, false, true) {
 		switch kv.id {
 		case "labels":
 			if expr := p.mayParseExpression(kv.val); expr != nil {
@@ -1130,7 +1172,7 @@ func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 	var stepsOnlyKey *String
 	var callOnlyKey *String
 
-	for _, kv := range p.parseMapping(fmt.Sprintf("%q job", id.Value), n, false) {
+	for _, kv := range p.parseMapping(fmt.Sprintf("%q job", id.Value), n, false, true) {
 		k, v := kv.key, kv.val
 		switch kv.id {
 		case "name":
@@ -1168,7 +1210,7 @@ func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 			ret.Steps = p.parseSteps(v)
 			stepsOnlyKey = k
 		case "timeout-minutes":
-			ret.TimeoutMinutes = p.parseFloat(v)
+			ret.TimeoutMinutes = p.parseTimeoutMinutes(v)
 			stepsOnlyKey = k
 		case "strategy":
 			ret.Strategy = p.parseStrategy(k.Pos, v)
@@ -1179,19 +1221,12 @@ func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 			ret.Container = p.parseContainer("container", k.Pos, v)
 			stepsOnlyKey = k
 		case "services":
-			services := p.parseSectionMapping("services", v, false)
-			ret.Services = make(map[string]*Service, len(services))
-			for _, s := range services {
-				ret.Services[s.id] = &Service{
-					Name:      s.key,
-					Container: p.parseContainer("services", s.key.Pos, s.val),
-				}
-			}
+			ret.Services = p.parseServices(v)
 		case "uses":
 			call.Uses = p.parseString(v, false)
 			callOnlyKey = k
 		case "with":
-			with := p.parseSectionMapping("with", v, false)
+			with := p.parseSectionMapping("with", v, false, false)
 			call.Inputs = make(map[string]*WorkflowCallInput, len(with))
 			for _, i := range with {
 				call.Inputs[i.id] = &WorkflowCallInput{
@@ -1210,7 +1245,7 @@ func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 					p.errorf(kv.val, "expected mapping node for secrets or \"inherit\" string node but found %q node", kv.val.Value)
 				}
 			} else {
-				secrets := p.parseSectionMapping("secrets", v, false)
+				secrets := p.parseSectionMapping("secrets", v, false, false)
 				call.Secrets = make(map[string]*WorkflowCallSecret, len(secrets))
 				for _, s := range secrets {
 					call.Secrets[s.id] = &WorkflowCallSecret{
@@ -1279,7 +1314,7 @@ func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 
 // https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#jobs
 func (p *parser) parseJobs(n *yaml.Node) map[string]*Job {
-	jobs := p.parseSectionMapping("jobs", n, false)
+	jobs := p.parseSectionMapping("jobs", n, false, false)
 	ret := make(map[string]*Job, len(jobs))
 	for _, kv := range jobs {
 		ret[kv.id] = p.parseJob(kv.key, kv.val)
@@ -1303,7 +1338,7 @@ func (p *parser) parse(n *yaml.Node) *Workflow {
 		return w
 	}
 
-	for _, kv := range p.parseMapping("workflow", n.Content[0], false) {
+	for _, kv := range p.parseMapping("workflow", n.Content[0], false, true) {
 		k, v := kv.key, kv.val
 		switch kv.id {
 		case "name":
@@ -1353,27 +1388,33 @@ func (p *parser) parse(n *yaml.Node) *Workflow {
 // 	}
 // }
 
-func handleYAMLError(err error) []*Error {
-	re := regexp.MustCompile(`\bline (\d+):`)
-
-	yamlErr := func(msg string) *Error {
-		l := 0
-		if ss := re.FindStringSubmatch(msg); len(ss) > 1 {
-			l, _ = strconv.Atoi(ss[1])
-		}
-		msg = fmt.Sprintf("could not parse as YAML: %s", msg)
-		return &Error{msg, "", l, 0, "yaml-syntax"}
-	}
-
+func handleYAMLUnmarshalError(err error) []*Error {
 	if te, ok := err.(*yaml.TypeError); ok {
 		errs := make([]*Error, 0, len(te.Errors))
-		for _, msg := range te.Errors {
-			errs = append(errs, yamlErr(msg))
+		for _, e := range te.Errors {
+			errs = append(errs, &Error{
+				Message: fmt.Sprintf("could not parse as YAML: %s", e.Err.Error()),
+				Line:    e.Line,
+				Column:  e.Column,
+				Kind:    "syntax-check",
+			})
 		}
 		return errs
 	}
 
-	return []*Error{yamlErr(err.Error())}
+	var m string
+	var l int
+	if pe, ok := err.(*yaml.ParserError); ok {
+		l = pe.Line
+		m = pe.Message
+	} else {
+		m = err.Error() // Fallback. I believe this line should be unreachable
+	}
+	return []*Error{&Error{
+		Message: fmt.Sprintf("could not parse as YAML: %s", m),
+		Kind:    "syntax-check",
+		Line:    l,
+	}}
 }
 
 // Parse parses given source as byte sequence into workflow syntax tree. It returns all errors
@@ -1383,7 +1424,7 @@ func Parse(b []byte) (*Workflow, []*Error) {
 	var n yaml.Node
 
 	if err := yaml.Unmarshal(b, &n); err != nil {
-		return nil, handleYAMLError(err)
+		return nil, handleYAMLUnmarshalError(err)
 	}
 
 	// Uncomment for checking YAML tree
