@@ -3,10 +3,14 @@ import { promises as fs } from 'fs';
 import { strict as assert } from 'assert';
 import { Crypto } from '@peculiar/webcrypto';
 
-globalThis.crypto = new Crypto();
+// This polyfill is necessary for Node.js v18 or earlier. `global.crypto` was added at v19.
+// https://github.com/nodejs/node/pull/42083/files
+if (typeof globalThis.crypto === 'undefined') {
+    globalThis.crypto = new Crypto();
+}
 
-// Inject global.Go
-require('./lib/js/wasm_exec.js');
+// Inject global.Go for testing `main.wasm`.
+require('./lib/js/wasm_exec.js'); // eslint-disable-line @typescript-eslint/no-require-imports
 
 class CheckResults {
     errors: ActionlintError[] | null = null;
@@ -59,7 +63,14 @@ jobs:
 
         const go = new Go();
         const bin = await fs.readFile('./main.wasm');
-        const result = await WebAssembly.instantiate(bin, go.importObject);
+        // `buffer` property is documented as `ArrayBuffer` but it is typed as `ArrayBufferLike`
+        // which is weaker than documented type. It confuses `WebAssembly.instantiate` call's result
+        // type and causes a type error.
+        const buf = bin.buffer as ArrayBuffer;
+        const result = await WebAssembly.instantiate(buf, go.importObject);
+
+        // Do not `await` this method call since it will never be settled
+        // eslint-disable-next-line @typescript-eslint/no-floating-promises
         go.run(result.instance);
     });
 
