@@ -2,6 +2,7 @@ package actionlint
 
 import (
 	"fmt"
+	"io"
 	"path/filepath"
 	"strings"
 	"testing"
@@ -241,8 +242,8 @@ func TestReusableWorkflowUnmarshalOK(t *testing.T) {
 				t.Fatal(err)
 			}
 
-			if !cmp.Equal(m, tc.want) {
-				t.Fatal("Parse result is unexpected. diff:\n" + cmp.Diff(m, tc.want))
+			if diff := cmp.Diff(m, tc.want); diff != "" {
+				t.Fatal("Parse result is unexpected. diff:\n" + diff)
 			}
 		})
 	}
@@ -362,8 +363,8 @@ func TestReusableWorkflowCacheFindMetadataOK(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	if !cmp.Equal(m, testReusableWorkflowWantedMetadata) {
-		t.Fatal(cmp.Diff(m, testReusableWorkflowWantedMetadata))
+	if diff := cmp.Diff(m, testReusableWorkflowWantedMetadata); diff != "" {
+		t.Fatal(diff)
 	}
 
 	m2, err := c.FindMetadata("./ok.yaml")
@@ -629,8 +630,8 @@ func TestReusableWorkflowMetadataFromASTNodeInputs(t *testing.T) {
 				t.Fatal("Event was not converted to event")
 			}
 
-			if !cmp.Equal(m.Inputs, tc.want) {
-				t.Error(cmp.Diff(m.Inputs, tc.want))
+			if diff := cmp.Diff(m.Inputs, tc.want); diff != "" {
+				t.Fatal(diff)
 			}
 			if len(m.Outputs) != 0 {
 				t.Error("Outputs are not empty", m.Outputs)
@@ -673,8 +674,8 @@ func TestReusableWorkflowMetadataFromASTNodeOutputs(t *testing.T) {
 				want[strings.ToLower(o)] = &ReusableWorkflowMetadataOutput{o}
 			}
 
-			if !cmp.Equal(m.Outputs, want) {
-				t.Error(cmp.Diff(m.Outputs, want))
+			if diff := cmp.Diff(m.Outputs, want); diff != "" {
+				t.Fatal(diff)
 			}
 			if len(m.Inputs) != 0 {
 				t.Error("Inputs are not empty", m.Inputs)
@@ -732,8 +733,8 @@ func TestReusableWorkflowMetadataFromASTNodeSecrets(t *testing.T) {
 				}
 			}
 
-			if !cmp.Equal(m.Secrets, want) {
-				t.Error(cmp.Diff(m.Secrets, want))
+			if diff := cmp.Diff(m.Secrets, want); diff != "" {
+				t.Fatal(diff)
 			}
 			if len(m.Inputs) != 0 {
 				t.Error("Inputs are not empty", m.Inputs)
@@ -810,8 +811,8 @@ func TestReusableWorkflowMetadataCacheFindOneMetadataConcurrently(t *testing.T)
 	}
 
 	for _, m := range ms {
-		if !cmp.Equal(testReusableWorkflowWantedMetadata, m) {
-			t.Fatal(cmp.Diff(testReusableWorkflowWantedMetadata, m))
+		if diff := cmp.Diff(testReusableWorkflowWantedMetadata, m); diff != "" {
+			t.Fatal(diff)
 		}
 	}
 
@@ -880,6 +881,22 @@ func TestReusableWorkflowMetadataCacheWriteFromFileAndASTNodeConcurrently(t *tes
 	}
 }
 
+func TestReusableWorkflowNullCache(t *testing.T) {
+	c := newNullLocalReusableWorkflowCache(io.Discard)
+	e := &WorkflowCallEvent{Inputs: []*WorkflowCallEventInput{}}
+
+	// This should do nothing
+	c.WriteWorkflowCallEvent(filepath.Join("foo", "test.yaml"), e)
+
+	m, err := c.FindMetadata("./foo/test.yaml")
+	if m != nil {
+		t.Errorf("metadata should never be found with null cache: %v", m)
+	}
+	if err != nil {
+		t.Errorf("error should not happen since the cache simply doesn't hit: %v", err)
+	}
+}
+
 func TestReusableWorkflowCacheFactory(t *testing.T) {
 	cwd := filepath.Join("path", "to", "project1")
 	f := NewLocalReusableWorkflowCacheFactory(cwd, nil)
@@ -897,4 +914,9 @@ func TestReusableWorkflowCacheFactory(t *testing.T) {
 	if c1 != c3 {
 		t.Errorf("Same cache was not used: %v vs %v", c1, c3)
 	}
+
+	c4 := f.GetCache(nil)
+	if c4.proj != nil {
+		t.Errorf("Null cache should be returned when project is nil: %v", c4)
+	}
 }
