@@ -5,13 +5,13 @@ import (
 	"strings"
 	"time"
 
-	"github.com/robfig/cron"
+	"github.com/robfig/cron/v3"
 )
 
 //go:generate go run ./scripts/generate-webhook-events ./all_webhooks.go
 
 // RuleEvents is a rule to check 'on' field in workflow.
-// https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows
+// https://docs.github.com/en/actions/reference/workflows-and-actions/events-that-trigger-workflows
 type RuleEvents struct {
 	RuleBase
 }
@@ -19,7 +19,10 @@ type RuleEvents struct {
 // NewRuleEvents creates new RuleEvents instance.
 func NewRuleEvents() *RuleEvents {
 	return &RuleEvents{
-		RuleBase: RuleBase{name: "events"},
+		RuleBase: RuleBase{
+			name: "events",
+			desc: "Checks for workflow trigger events at \"on:\"",
+		},
 	}
 }
 
@@ -55,7 +58,7 @@ func (rule *RuleEvents) checkCron(spec *String) {
 	p := cron.NewParser(cron.Minute | cron.Hour | cron.Dom | cron.Month | cron.Dow)
 	sched, err := p.Parse(spec.Value)
 	if err != nil {
-		rule.errorf(spec.Pos, "invalid CRON format %q in schedule event: %s", spec.Value, err.Error())
+		rule.Errorf(spec.Pos, "invalid CRON format %q in schedule event: %s", spec.Value, err.Error())
 		return
 	}
 
@@ -63,11 +66,11 @@ func (rule *RuleEvents) checkCron(spec *String) {
 	next := sched.Next(start)
 	diff := next.Sub(start).Seconds()
 
-	// (#14) https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows#scheduled-events
+	// (#14) https://docs.github.com/en/actions/reference/workflows-and-actions/events-that-trigger-workflows#scheduled-events
 	//
 	// > The shortest interval you can run scheduled workflows is once every 5 minutes.
 	if diff < 60.0*5 {
-		rule.errorf(spec.Pos, "scheduled job runs too frequently. it runs once per %g seconds. the shortest interval is once every 5 minutes", diff)
+		rule.Errorf(spec.Pos, "scheduled job runs too frequently. it runs once per %g seconds. the shortest interval is once every 5 minutes", diff)
 	}
 }
 
@@ -76,7 +79,7 @@ func (rule *RuleEvents) filterNotAvailable(pos *Pos, filter, hook string, availa
 	if len(available) < 2 {
 		e = "event"
 	}
-	rule.errorf(pos, "%q filter is not available for %s event. it is only for %s %s", filter, hook, strings.Join(available, ", "), e)
+	rule.Errorf(pos, "%q filter is not available for %s event. it is only for %s %s", filter, hook, strings.Join(available, ", "), e)
 }
 
 func (rule *RuleEvents) checkExclusiveFilters(filter, ignore *WebhookEventFilter, hook string, available []string) {
@@ -94,7 +97,7 @@ func (rule *RuleEvents) checkExclusiveFilters(filter, ignore *WebhookEventFilter
 			if p.IsBefore(ignore.Name.Pos) {
 				p = ignore.Name.Pos
 			}
-			rule.errorf(p, "both %q and %q filters cannot be used for the same event %q. note: use '!' to negate patterns", filter.Name.Value, ignore.Name.Value, hook)
+			rule.Errorf(p, "both %q and %q filters cannot be used for the same event %q. note: use '!' to negate patterns", filter.Name.Value, ignore.Name.Value, hook)
 		}
 	} else {
 		if !filter.IsEmpty() {
@@ -106,13 +109,13 @@ func (rule *RuleEvents) checkExclusiveFilters(filter, ignore *WebhookEventFilter
 	}
 }
 
-// https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows#webhook-events
+// https://docs.github.com/en/actions/reference/workflows-and-actions/events-that-trigger-workflows#webhook-events
 func (rule *RuleEvents) checkWebhookEvent(event *WebhookEvent) {
 	hook := event.Hook.Value
 
 	types, ok := AllWebhookTypes[hook]
 	if !ok {
-		rule.errorf(event.Pos, "unknown Webhook event %q. see https://docs.github.com/en/actions/learn-github-actions/events-that-trigger-workflows#webhook-events for list of all Webhook event names", hook)
+		rule.Errorf(event.Pos, "unknown Webhook event %q. see https://docs.github.com/en/actions/reference/workflows-and-actions/events-that-trigger-workflows#webhook-events for list of all Webhook event names", hook)
 		return
 	}
 
@@ -120,15 +123,16 @@ func (rule *RuleEvents) checkWebhookEvent(event *WebhookEvent) {
 
 	if hook == "workflow_run" {
 		if len(event.Workflows) == 0 {
-			rule.error(event.Pos, "no workflow is configured for \"workflow_run\" event")
+			rule.Error(event.Pos, "no workflow is configured for \"workflow_run\" event")
 		}
 	} else {
 		if len(event.Workflows) != 0 {
-			rule.errorf(event.Pos, "\"workflows\" cannot be configured for %q event. it is only for workflow_run event", hook)
+			rule.Errorf(event.Pos, "\"workflows\" cannot be configured for %q event. it is only for workflow_run event", hook)
 		}
 	}
 
 	// Some filters are available with specific events and exclusive
+	// - on.merge_group.<branches|branches-ignore>
 	// - on.<push|pull_request|pull_request_target>.<paths|paths-ignore>
 	// - on.push.<branches|tags|branches-ignore|tags-ignore>
 	// - on.<pull_request|pull_request_target>.<branches|branches-ignore>
@@ -143,7 +147,7 @@ func (rule *RuleEvents) checkWebhookEvent(event *WebhookEvent) {
 		event.Branches,
 		event.BranchesIgnore,
 		hook,
-		[]string{"push", "pull_request", "pull_request_target", "workflow_run"},
+		[]string{"merge_group", "push", "pull_request", "pull_request_target", "workflow_run"},
 	)
 	rule.checkExclusiveFilters(
 		event.Tags,
@@ -155,7 +159,7 @@ func (rule *RuleEvents) checkWebhookEvent(event *WebhookEvent) {
 
 func (rule *RuleEvents) checkTypes(hook *String, types []*String, expected []string) {
 	if len(expected) == 0 && len(types) > 0 {
-		rule.errorf(hook.Pos, "\"types\" cannot be specified for %q Webhook event", hook.Value)
+		rule.Errorf(hook.Pos, "\"types\" cannot be specified for %q Webhook event", hook.Value)
 		return
 	}
 
@@ -168,7 +172,7 @@ func (rule *RuleEvents) checkTypes(hook *String, types []*String, expected []str
 			}
 		}
 		if !valid {
-			rule.errorf(
+			rule.Errorf(
 				ty.Pos,
 				"invalid activity type %q for %q Webhook event. available types are %s",
 				ty.Value,
@@ -186,11 +190,11 @@ func (rule *RuleEvents) checkWorkflowCallEvent(event *WorkflowCallEvent) {
 			continue
 		}
 		// ${{ }} is available in the default value
-		if !strings.Contains(i.Default.Value, "${{") {
+		if !i.Default.ContainsExpression() {
 			switch i.Type {
 			case WorkflowCallEventInputTypeNumber:
 				if _, err := strconv.ParseFloat(i.Default.Value, 64); err != nil {
-					rule.errorf(
+					rule.Errorf(
 						i.Default.Pos,
 						"input of workflow_call event %q is typed as number but its default value %q cannot be parsed as a float number: %s",
 						i.Name.Value,
@@ -200,7 +204,7 @@ func (rule *RuleEvents) checkWorkflowCallEvent(event *WorkflowCallEvent) {
 				}
 			case WorkflowCallEventInputTypeBoolean:
 				if d := strings.ToLower(i.Default.Value); d != "true" && d != "false" {
-					rule.errorf(
+					rule.Errorf(
 						i.Default.Pos,
 						"input of workflow_call event %q is typed as boolean. its default value must be true or false but got %q",
 						i.Name.Value,
@@ -210,7 +214,7 @@ func (rule *RuleEvents) checkWorkflowCallEvent(event *WorkflowCallEvent) {
 			}
 		}
 		if i.IsRequired() {
-			rule.errorf(
+			rule.Errorf(
 				i.Default.Pos,
 				"input %q of workflow_call event has the default value %q, but it is also required. if an input is marked as required, its default value will never be used",
 				i.Name.Value,
@@ -226,13 +230,13 @@ func (rule *RuleEvents) checkWorkflowDispatchEvent(event *WorkflowDispatchEvent)
 	for n, i := range event.Inputs {
 		if i.Type == WorkflowDispatchEventInputTypeChoice {
 			if len(i.Options) == 0 {
-				rule.errorf(i.Name.Pos, "input type of %q is \"choice\" but \"options\" is not set", n)
+				rule.Errorf(i.Name.Pos, "input type of %q is \"choice\" but \"options\" is not set", n)
 				continue
 			}
 			seen := make(map[string]struct{}, len(i.Options))
 			for _, o := range i.Options {
 				if _, ok := seen[o.Value]; ok {
-					rule.errorf(o.Pos, "option %q is duplicated in options of %q input", o.Value, n)
+					rule.Errorf(o.Pos, "option %q is duplicated in options of %q input", o.Value, n)
 					continue
 				}
 				seen[o.Value] = struct{}{}
@@ -243,24 +247,42 @@ func (rule *RuleEvents) checkWorkflowDispatchEvent(event *WorkflowDispatchEvent)
 					b.append(o.Value)
 				}
 				if _, ok := seen[i.Default.Value]; !ok {
-					rule.errorf(i.Default.Pos, "default value %q of %q input is not included in its options %q", i.Default.Value, n, b.build())
+					rule.Errorf(i.Default.Pos, "default value %q of %q input is not included in its options %q", i.Default.Value, n, b.build())
 				}
 			}
 		} else {
 			if len(i.Options) > 0 {
-				rule.errorf(i.Name.Pos, "\"options\" can not be set to %q input because its input type is not \"choice\"", n)
+				rule.Errorf(i.Name.Pos, "\"options\" can not be set to %q input because its input type is not \"choice\"", n)
 			}
-			switch i.Type {
-			case WorkflowDispatchEventInputTypeBoolean:
-				if i.Default != nil {
-					if d := strings.ToLower(i.Default.Value); d != "true" && d != "false" {
-						rule.errorf(i.Default.Pos, "type of %q input is \"boolean\". its default value %q must be \"true\" or \"false\"", n, i.Default.Value)
-					}
-				}
-			default:
+			if i.Default != nil {
 				// TODO: Can some check be done for WorkflowDispatchEventInputTypeEnvironment?
 				// What is suitable for default value of the type? (Or is a default value never suitable?)
+				switch i.Type {
+				case WorkflowDispatchEventInputTypeNumber:
+					if _, err := strconv.ParseFloat(i.Default.Value, 64); err != nil {
+						rule.Errorf(
+							i.Default.Pos,
+							"type of %q input is \"number\" but its default value %q cannot be parsed as a float number: %s",
+							i.Name.Value,
+							i.Default.Value,
+							err,
+						)
+					}
+				case WorkflowDispatchEventInputTypeBoolean:
+					if d := strings.ToLower(i.Default.Value); d != "true" && d != "false" {
+						rule.Errorf(i.Default.Pos, "type of %q input is \"boolean\". its default value %q must be \"true\" or \"false\"", n, i.Default.Value)
+					}
+				}
 			}
 		}
 	}
+	// Maximum number of inputs is 10
+	// https://docs.github.com/en/actions/using-workflows/events-that-trigger-workflows#providing-inputs
+	if len(event.Inputs) > 10 {
+		rule.Errorf(
+			event.Pos,
+			"maximum number of inputs for \"workflow_dispatch\" event is 10 but %d inputs are provided. see https://docs.github.com/en/actions/using-workflows/events-that-trigger-workflows#providing-inputs",
+			len(event.Inputs),
+		)
+	}
 }
