@@ -6,9 +6,10 @@ var allPermissionScopes = map[string]struct{}{
 	"checks":              {},
 	"contents":            {},
 	"deployments":         {},
+	"discussions":         {},
 	"id-token":            {},
 	"issues":              {},
-	"discussions":         {},
+	"models":              {},
 	"packages":            {},
 	"pages":               {},
 	"pull-requests":       {},
@@ -18,7 +19,7 @@ var allPermissionScopes = map[string]struct{}{
 }
 
 // RulePermissions is a rule checker to check permission configurations in a workflow.
-// https://docs.github.com/en/actions/security-guides/automatic-token-authentication#permissions-for-the-github_token
+// https://docs.github.com/en/actions/security-for-github-actions/security-guides/automatic-token-authentication#permissions-for-the-github_token
 type RulePermissions struct {
 	RuleBase
 }
