@@ -16,7 +16,7 @@ func TestRuleRunnerLabelCheckLabels(t *testing.T) {
 		// Normal cases
 		{
 			what:   "simple GH-hosted Linux runner label",
-			labels: []string{"ubuntu-20.04"},
+			labels: []string{"ubuntu-24.04"},
 		},
 		{
 			what:   "simple GH-hosted Windows runner label",
@@ -24,11 +24,11 @@ func TestRuleRunnerLabelCheckLabels(t *testing.T) {
 		},
 		{
 			what:   "simple GH-hosted macOS runner label",
-			labels: []string{"macos-11.0"},
+			labels: []string{"macos-14"},
 		},
 		{
 			what:   "simple GH-hosted runner label in upper case",
-			labels: []string{"macOS-11"},
+			labels: []string{"macOS-14"},
 		},
 		{
 			what:   "self-hosted Linux runner",
@@ -36,11 +36,15 @@ func TestRuleRunnerLabelCheckLabels(t *testing.T) {
 		},
 		{
 			what:   "self-hosted all Linux runner labels",
-			labels: []string{"self-hosted", "linux", "ubuntu-22.04", "ubuntu-latest"},
+			labels: []string{"self-hosted", "linux", "ubuntu-24.04", "ubuntu-latest"},
 		},
 		{
 			what:   "self-hosted all macOS runner labels",
-			labels: []string{"self-hosted", "macOS", "macOS-latest", "macOS-14.0", "macOS-14"},
+			labels: []string{"self-hosted", "macOS", "macOS-latest", "macOS-15"},
+		},
+		{
+			what:   "self-hosted all Windows runner labels",
+			labels: []string{"self-hosted", "windows", "windows-latest", "windows-2022"},
 		},
 		{
 			what:   "self-hosted Linux runner in upper case",
@@ -52,7 +56,7 @@ func TestRuleRunnerLabelCheckLabels(t *testing.T) {
 		},
 		{
 			what:   "self-hosted runner with GH-hosted runner label",
-			labels: []string{"self-hosted", "ubuntu-20.04"},
+			labels: []string{"self-hosted", "ubuntu-24.04"},
 		},
 		{
 			what:   "larger Ubuntu runner",
@@ -67,8 +71,16 @@ func TestRuleRunnerLabelCheckLabels(t *testing.T) {
 			labels: []string{"windows-latest-8-cores"},
 		},
 		{
-			what:   "multiple labels for GH-hosted runner",
-			labels: []string{"ubuntu-latest", "ubuntu-22.04"},
+			what:   "multiple labels for GH-hosted Linux runner",
+			labels: []string{"ubuntu-latest", "ubuntu-24.04"},
+		},
+		{
+			what:   "multiple labels for GH-hosted Windows runner",
+			labels: []string{"windows-latest", "windows-2022"},
+		},
+		{
+			what:   "multiple labels for GH-hosted macOS runner",
+			labels: []string{"macos-latest", "macos-15"},
 		},
 		{
 			what:   "user-defined labels",
@@ -97,7 +109,7 @@ func TestRuleRunnerLabelCheckLabels(t *testing.T) {
 		},
 		{
 			what:   "matrix at first label",
-			labels: []string{"${{matrix.os}}", "ubuntu-22.04"},
+			labels: []string{"${{matrix.os}}", "ubuntu-24.04"},
 			matrix: []string{"ubuntu-latest"},
 		},
 		{
@@ -145,6 +157,10 @@ func TestRuleRunnerLabelCheckLabels(t *testing.T) {
 			what:   "use matrix value but no matrix exist",
 			labels: []string{"${{matrix.os}}"},
 		},
+		{
+			what:   "ubuntu-24.04",
+			labels: []string{"ubuntu-24.04"},
+		},
 		// TODO: Add tests for 'include:'
 		// TODO: Check matrix with 'include:'
 
@@ -218,14 +234,34 @@ func TestRuleRunnerLabelCheckLabels(t *testing.T) {
 			},
 		},
 		{
-			what:   "Linux labels conflict",
-			labels: []string{"ubuntu-latest", "ubuntu-20.04"},
-			errs:   []string{`label "ubuntu-20.04" conflicts with label "ubuntu-latest"`},
+			what:   "Linux labels version conflict",
+			labels: []string{"ubuntu-latest", "ubuntu-22.04"},
+			errs:   []string{`label "ubuntu-22.04" conflicts with label "ubuntu-latest"`},
 		},
 		{
-			what:   "macOS labels conflict",
-			labels: []string{"macos-11", "macos-12"},
-			errs:   []string{`label "macos-12" conflicts with label "macos-11"`},
+			what:   "macOS labels version conflict",
+			labels: []string{"macos-14", "macos-15"},
+			errs:   []string{`label "macos-15" conflicts with label "macos-14"`},
+		},
+		{
+			what:   "Windows labels version conflict",
+			labels: []string{"windows-2025", "windows-2022"},
+			errs:   []string{`label "windows-2022" conflicts with label "windows-2025"`},
+		},
+		{
+			what:   "Linux labels architecture conflict",
+			labels: []string{"ubuntu-24.04", "ubuntu-22.04"},
+			errs:   []string{`label "ubuntu-22.04" conflicts with label "ubuntu-24.04"`},
+		},
+		{
+			what:   "macOS labels architecture conflict",
+			labels: []string{"macos-15", "macos-15-intel"},
+			errs:   []string{`label "macos-15-intel" conflicts with label "macos-15"`},
+		},
+		{
+			what:   "Windows labels architecture conflict",
+			labels: []string{"windows-2025", "windows-11-arm"},
+			errs:   []string{`label "windows-11-arm" conflicts with label "windows-2025"`},
 		},
 		{
 			what:   "macOS XL and normal labels conflict",
